package eu.e99.svc.common.protocol;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Supplier;

public class Packets {

    /**
     * This number is incremented with every change to the protocol.
     */
    public static final int VERSION = 0;

    private static final Map<Integer, Supplier<? extends Packet>> PACKETS_BY_ID = new HashMap<>();
    private static final Map<Class<? extends Packet>, Integer> ID_BY_PACKETS = new HashMap<>();

    static {
        registerPacket(0, DisconnectPacket::new);
        registerPacket(1, HandshakePacket::new);
        registerPacket(2, AuthRequestPacket::new);
        registerPacket(3, AuthResponsePacket::new);
        registerPacket(4, AuthSuccessPacket::new);
    }

    private static void registerPacket(int id, Supplier<? extends Packet> supplier) {
        Packet packet = supplier.get();
        Class<? extends Packet> clazz = packet.getClass();


        PACKETS_BY_ID.put(id, supplier);
        ID_BY_PACKETS.put(clazz, id);
    }

    public static void write(Packet packet, DataOutputStream out) throws IOException {
        Class<? extends Packet> clazz = packet.getClass();
        Integer id = ID_BY_PACKETS.get(clazz);
        if (id == null) {
            throw new IllegalArgumentException("Unknown packet");
        }

        out.writeInt(id);
        packet.write(out);
    }

    public static Packet read(DataInputStream in) throws IOException {
        int id = in.readInt();
        Supplier<? extends Packet> supplier = PACKETS_BY_ID.get(id);
        if (supplier == null) {
            return null;
        }

        Packet packet = supplier.get();
        packet.read(in);
        return packet;
    }
}
